# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class LocalizationType(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attribute_types': 'list[AttributeType]',
        'color_map': 'ColorMap',
        'description': 'str',
        'dtype': 'str',
        'grouping_default': 'bool',
        'id': 'int',
        'line_width': 'int',
        'media': 'list[int]',
        'name': 'str',
        'project': 'int',
        'visible': 'bool'
    }

    attribute_map = {
        'attribute_types': 'attribute_types',
        'color_map': 'colorMap',
        'description': 'description',
        'dtype': 'dtype',
        'grouping_default': 'grouping_default',
        'id': 'id',
        'line_width': 'line_width',
        'media': 'media',
        'name': 'name',
        'project': 'project',
        'visible': 'visible'
    }

    def __init__(self, attribute_types=None, color_map=None, description=None, dtype=None, grouping_default=True, id=None, line_width=None, media=None, name=None, project=None, visible=True, local_vars_configuration=None):  # noqa: E501
        """LocalizationType - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attribute_types = None
        self._color_map = None
        self._description = None
        self._dtype = None
        self._grouping_default = None
        self._id = None
        self._line_width = None
        self._media = None
        self._name = None
        self._project = None
        self._visible = None
        self.discriminator = None

        if attribute_types is not None:
            self.attribute_types = attribute_types
        if color_map is not None:
            self.color_map = color_map
        if description is not None:
            self.description = description
        if dtype is not None:
            self.dtype = dtype
        if grouping_default is not None:
            self.grouping_default = grouping_default
        if id is not None:
            self.id = id
        if line_width is not None:
            self.line_width = line_width
        if media is not None:
            self.media = media
        if name is not None:
            self.name = name
        if project is not None:
            self.project = project
        if visible is not None:
            self.visible = visible

    @property
    def attribute_types(self):
        """
        Attribute type definitions.

        :return: The attribute_types of this LocalizationType. 
        :rtype: list[AttributeType]
        """
        return self._attribute_types

    @attribute_types.setter
    def attribute_types(self, attribute_types):
        """
        Attribute type definitions.

        :param attribute_types: The attribute_types of this LocalizationType.
        :type: list[AttributeType]
        """

        self._attribute_types = attribute_types

    @property
    def color_map(self):
        """

        :return: The color_map of this LocalizationType. 
        :rtype: ColorMap
        """
        return self._color_map

    @color_map.setter
    def color_map(self, color_map):
        """

        :param color_map: The color_map of this LocalizationType.
        :type: ColorMap
        """

        self._color_map = color_map

    @property
    def description(self):
        """
        Description of the localization type.

        :return: The description of this LocalizationType. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the localization type.

        :param description: The description of this LocalizationType.
        :type: str
        """

        self._description = description

    @property
    def dtype(self):
        """
        Shape of this localization type.

        :return: The dtype of this LocalizationType. 
        :rtype: str
        """
        return self._dtype

    @dtype.setter
    def dtype(self, dtype):
        """
        Shape of this localization type.

        :param dtype: The dtype of this LocalizationType.
        :type: str
        """
        allowed_values = ["box", "line", "dot"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and dtype not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `dtype` ({0}), must be one of {1}"  # noqa: E501
                .format(dtype, allowed_values)
            )

        self._dtype = dtype

    @property
    def grouping_default(self):
        """
        Whether to group elements in the UI by default.

        :return: The grouping_default of this LocalizationType. 
        :rtype: bool
        """
        return self._grouping_default

    @grouping_default.setter
    def grouping_default(self, grouping_default):
        """
        Whether to group elements in the UI by default.

        :param grouping_default: The grouping_default of this LocalizationType.
        :type: bool
        """

        self._grouping_default = grouping_default

    @property
    def id(self):
        """
        Unique integer identifying a localization type.

        :return: The id of this LocalizationType. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying a localization type.

        :param id: The id of this LocalizationType.
        :type: int
        """

        self._id = id

    @property
    def line_width(self):
        """
        Width of the line used to draw the localization.

        :return: The line_width of this LocalizationType. 
        :rtype: int
        """
        return self._line_width

    @line_width.setter
    def line_width(self, line_width):
        """
        Width of the line used to draw the localization.

        :param line_width: The line_width of this LocalizationType.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                line_width is not None and line_width < 1):  # noqa: E501
            raise ValueError("Invalid value for `line_width`, must be a value greater than or equal to `1`")  # noqa: E501

        self._line_width = line_width

    @property
    def media(self):
        """
        List of integers identifying media types that this localization type may apply to.

        :return: The media of this LocalizationType. 
        :rtype: list[int]
        """
        return self._media

    @media.setter
    def media(self, media):
        """
        List of integers identifying media types that this localization type may apply to.

        :param media: The media of this LocalizationType.
        :type: list[int]
        """

        self._media = media

    @property
    def name(self):
        """
        Name of the localization type.

        :return: The name of this LocalizationType. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the localization type.

        :param name: The name of this LocalizationType.
        :type: str
        """

        self._name = name

    @property
    def project(self):
        """
        Unique integer identifying project for this leaf type.

        :return: The project of this LocalizationType. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying project for this leaf type.

        :param project: The project of this LocalizationType.
        :type: int
        """

        self._project = project

    @property
    def visible(self):
        """
        Whether this type should be displayed in the UI.

        :return: The visible of this LocalizationType. 
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """
        Whether this type should be displayed in the UI.

        :param visible: The visible of this LocalizationType.
        :type: bool
        """

        self._visible = visible

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LocalizationType):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LocalizationType):
            return True

        return self.to_dict() != other.to_dict()



import tempfile
from time import sleep
import os

import tator

def test_get_file(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    with tempfile.TemporaryDirectory() as temp_dir:
        outpath = os.path.join(temp_dir, "video.mp4")
        for progress in tator.download_media(tator_api, video_obj, outpath):
            print(f"Video download progress: {progress}%")
        assert(os.path.exists(outpath))

def test_get_audio(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    audio = video_obj.media_files.audio
    assert len(audio) > 0
    assert audio[0].codec == 'aac'

def test_get_by_id(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    other_obj = tator_api.get_media_list_by_id(project, {'ids': [video]})[0]
    assert video_obj.id == other_obj.id
    other_obj = tator_api.get_media_list_by_id(project, {'ids': [video]}, force_es=1)[0]
    assert video_obj.id == other_obj.id


def test_archive(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    # Test default value of `archived` is "live"
    assert video_obj.archive_state == "live"

    # Test default `get_media_list` filters on `archive_lifecycle == "live"`
    response = tator_api.get_media_list(project, media_id=[video])
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Test `get_media_list` with `archive_lifecycle="archived"` doesn't return "live" objects
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="archived")
    assert len(response) == 0

    # Test returning subset of media that is live
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="live")
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Test returning subset of media that has any `archive_lifecycle` state
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="all")
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Mark the video to archive
    tator_api.update_media_list(project, {"archive_state": "to_archive", "ids": [video]})

    # Wait for update to propagate to ES
    sleep(2)

    # Test default `get_media_list` filters on `archive_lifecycle == "live"`
    response = tator_api.get_media_list(project, media_id=[video])
    assert len(response) == 0

    # Test `get_media_list` with `archive_lifecycle="archived"` returns archived objects
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="archived")
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Test with force_es
    response = tator_api.get_media_list(
        project, media_id=[video], archive_lifecycle="archived", force_es=1
    )
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Test returning subset of media that is live
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="live")
    assert len(response) == 0

    # Test with force_es
    response = tator_api.get_media_list(
        project, media_id=[video], archive_lifecycle="live", force_es=1
    )
    assert len(response) == 0

    # Test returning subset of media that has any `archive_state` state
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="all")
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Test with force_es
    response = tator_api.get_media_list(
        project, media_id=[video], archive_lifecycle="all", force_es=1
    )
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Additional attempts to set the state to `to_archive` will not change anything
    tator_api.update_media(video, {"archive_state": "to_archive"})
    video_obj = tator_api.get_media(video)
    assert video_obj.archive_state == "to_archive"

    # Setting the `archive_state` to `to_live` when a media is in the state `to_archive` should
    # result in the media's state changing to `live`
    tator_api.update_media(video, {"archive_state": "to_live"})
    video_obj = tator_api.get_media(video)
    assert video_obj.archive_state == "live"



from pprint import pformat
import datetime
import random
from time import sleep
import uuid

import tator
from ._common import assert_close_enough


def wait_for_parity(tator_api, project, patch, expected_ids):
    attribute_filter = [f"test_string::{patch['attributes']['test_string']}"]
    state_id_query = {"ids": expected_ids}
    total_timeout = 60.0 # seconds
    wait_time = 2.0 # seconds

    print(f"Using attribute filter:\n{pformat(attribute_filter)}")
    from_psql = tator_api.get_state_list_by_id(
        project,
        localization_id_query=state_id_query,
        attribute=attribute_filter,
    )
    for idx in range(int(total_timeout / wait_time) + 1):
        from_es = tator_api.get_state_list_by_id(
            project,
            localization_id_query=state_id_query,
            attribute=attribute_filter,
            force_es=1,
        )

        if len(from_es) == len(from_psql):
            print(f"Found expected number of results after {idx * wait_time} seconds")
            return True

        sleep(wait_time)

    print(f"Did not find expected number of results after {idx * wait_time} seconds")
    return False

def random_state(project, state_type, video_obj, post=False):
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    out = {
        'project': project,
        'type': state_type,
        'media_ids': [video_obj.id],
        'frame': random.randint(0, video_obj.num_frames - 1),
    }
    if post:
        out = {**out, **attributes}
    else:
        out['attributes'] = attributes
    return out

def comparison_query(tator_api, project, state_ids, exclude):
    """ Runs a random query and compares results with ES enabled and disabled.
    """
    bool_value = random.choice([True, False])
    int_lower = random.randint(-1000, 0)
    int_upper = random.randint(0, 1000)
    float_lower = random.uniform(-1000.0, 0.0)
    float_upper = random.uniform(0.0, 1000.0)
    enum_value = random.choice(['a', 'b', 'c'])
    state_id_query = {"ids": state_ids}
    attribute_filter = [f"test_bool::{'true' if bool_value else 'false'}", f"test_enum::{enum_value}"]
    attribute_lte_filter = [f"test_int::{int_upper}"]
    attribute_gte_filter = [f"test_int::{int_lower}"]
    attribute_lt_filter = [f"test_float::{float_upper}"]
    attribute_gt_filter = [f"test_float::{float_lower}"]
    t0 = datetime.datetime.now()
    from_psql = tator_api.get_state_list_by_id(
        project,
        localization_id_query=state_id_query,
        attribute=attribute_filter,
        attribute_lte=attribute_lte_filter,
        attribute_gte=attribute_gte_filter,
        attribute_lt=attribute_lt_filter,
        attribute_gt=attribute_gt_filter,
    )
    psql_time = datetime.datetime.now() - t0
    t0 = datetime.datetime.now()
    from_es = tator_api.get_state_list_by_id(
        project,
        localization_id_query=state_id_query,
        attribute=attribute_filter,
        attribute_lte=attribute_lte_filter,
        attribute_gte=attribute_gte_filter,
        attribute_lt=attribute_lt_filter,
        attribute_gt=attribute_gt_filter,
        force_es=1,
    )
    es_time = datetime.datetime.now() - t0

    not_in_es = [psql_ele.id for psql_ele in from_psql if psql_ele not in from_es]
    not_in_psql = [es_ele.id for es_ele in from_es if es_ele not in from_psql]
    print(f"Found {len(not_in_es)} results in PSQL that were not in ES\n{pformat(not_in_es)}")
    print(f"Found {len(not_in_psql)} results in ES that were not in PSQL\n{pformat(not_in_psql)}")

    assert len(from_psql) == len(from_es)
    for psql, es in zip(from_psql, from_es):
        assert_close_enough(psql, es, exclude)
        assert(psql.attributes['test_bool'] == bool_value)
        assert(psql.attributes['test_int'] <= int_upper)
        assert(psql.attributes['test_int'] >= int_lower)
        assert(psql.attributes['test_float'] < float_upper)
        assert(psql.attributes['test_float'] > float_lower)
        assert(psql.attributes['test_enum'] == enum_value)
    return psql_time, es_time

def test_state_crud(host, token, project, video_type, video, state_type):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    # These fields will not be checked for object equivalence after patch.
    exclude = ['project', 'type', 'media_ids', 'id', 'meta', 'user', 'frame', 'ids']

    # Test bulk create.
    num_states = random.randint(2000, 10000)
    states = [
        random_state(project, state_type, video_obj, post=True)
        for _ in range(num_states)
    ]
    state_ids = []
    for response in tator.util.chunked_create(tator_api.create_state_list,
                                         project, state_spec=states):
        state_ids += response.id
    assert len(state_ids) == len(states)
    print(f"Created {len(state_ids)} states!")

    # Verify list contains the number of entities created
    response = tator_api.get_state_list(project, type=state_type)
    assert len(response) == num_states

    # Test media retrieval by state ID.
    response = tator_api.get_media_list_by_id(project, {'state_ids': state_ids})
    assert len(response) == 1
    assert response[0].id == video
    response = tator_api.get_media_list_by_id(project, {'state_ids': state_ids}, force_es=1)
    assert len(response) == 1
    assert response[0].id == video

    # Test state retrival by media ID.
    response = tator_api.get_state_list_by_id(project, {'media_ids': [video]})
    assert(len(response) == len(state_ids))
    response = tator_api.get_state_list_by_id(project, {'media_ids': [video]}, force_es=1)
    assert(len(response) == len(state_ids))

    # Test single create.
    state = random_state(project, state_type, video_obj, post=True)
    response = tator_api.create_state_list(project, state_spec=[state])
    assert isinstance(response, tator.models.CreateListResponse)
    print(response.message)
    state_id = response.id[0]

    # Patch single state.
    patch = random_state(project, state_type, video_obj)
    response = tator_api.update_state(state_id, state_update=patch)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Get single state.
    updated_state = tator_api.get_state(state_id)
    assert isinstance(updated_state, tator.models.State)
    assert_close_enough(patch, updated_state, exclude)

    # Get state by ID.
    state_by_id = tator_api.get_state_list_by_id(project, {'ids': [state_id]})
    assert(len(state_by_id) == 1)
    state_by_id = state_by_id[0]
    assert_close_enough(updated_state, state_by_id, exclude)

    # Delete single state.
    response = tator_api.delete_state(state_id)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    params = {'media_id': [video], 'type': state_type}
    assert(tator_api.get_state_count(project, **params) == len(states))

    # Bulk update state attributes.
    bulk_patch = random_state(project, state_type, video_obj)
    bulk_patch = {'attributes': bulk_patch['attributes']}
    response = tator_api.update_state_list(project, **params,
                                           state_bulk_update=bulk_patch)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Bulk update specified states by ID.
    id_bulk_patch = random_state(project, state_type, video_obj)
    update_ids = random.choices(state_ids, k=100)
    id_bulk_patch = {'attributes': id_bulk_patch['attributes'], 'ids': update_ids}
    response = tator_api.update_state_list(project, **params,
                                           state_bulk_update=id_bulk_patch)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Verify all states have been updated.
    states = tator_api.get_state_list(project, **params)
    dataframe = tator.util.to_dataframe(states)
    assert(len(states)==len(dataframe))
    for state in states:
        if state.id in update_ids:
            assert_close_enough(id_bulk_patch, state, exclude)
        else:
            assert_close_enough(bulk_patch, state, exclude)

    # Do random queries using psql and elasticsearch and compare results.
    assert wait_for_parity(tator_api, project, id_bulk_patch, update_ids)
    es_time = datetime.timedelta(seconds=0)
    psql_time = datetime.timedelta(seconds=0)
    state_ids = [state.id for state in states]
    NUM_QUERIES = 10
    for _ in range(NUM_QUERIES):
        psql, es = comparison_query(tator_api, project, state_ids, exclude)
        psql_time += psql
        es_time += es
    print(f"Over {NUM_QUERIES} random attribute queries:")
    print(f"  Avg PSQL time: {psql_time / NUM_QUERIES}")
    print(f"  Avg ES time: {es_time / NUM_QUERIES}")

    # Clone states to same media.
    version_mapping = {version.id: version.id for version in tator_api.get_version_list(project)}
    generator = tator.util.clone_state_list(tator_api, {**params, 'project': project},
                                            project, version_mapping, {video:video}, {},
                                            {state_type:state_type}, tator_api)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} states...")
    print(f"Finished creating {num_created} states!")
    assert(tator_api.get_state_count(project, **params) == 2 * len(states))

    # Delete all states.
    response = tator_api.delete_state_list(project, **params)
    assert isinstance(response, tator.models.MessageResponse)

    # Verify all states are gone.
    states = tator_api.get_state_list(project, **params)
    assert states == []



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Bucket(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'access_key': 'str',
        'endpoint_url': 'str',
        'id': 'int',
        'name': 'str',
        'organization': 'int',
        'region': 'str',
        'secret_key': 'str'
    }

    attribute_map = {
        'access_key': 'access_key',
        'endpoint_url': 'endpoint_url',
        'id': 'id',
        'name': 'name',
        'organization': 'organization',
        'region': 'region',
        'secret_key': 'secret_key'
    }

    def __init__(self, access_key=None, endpoint_url=None, id=None, name=None, organization=None, region=None, secret_key=None, local_vars_configuration=None):  # noqa: E501
        """Bucket - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._access_key = None
        self._endpoint_url = None
        self._id = None
        self._name = None
        self._organization = None
        self._region = None
        self._secret_key = None
        self.discriminator = None

        if access_key is not None:
            self.access_key = access_key
        if endpoint_url is not None:
            self.endpoint_url = endpoint_url
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if organization is not None:
            self.organization = organization
        if region is not None:
            self.region = region
        if secret_key is not None:
            self.secret_key = secret_key

    @property
    def access_key(self):
        """
        Account access key.

        :return: The access_key of this Bucket. 
        :rtype: str
        """
        return self._access_key

    @access_key.setter
    def access_key(self, access_key):
        """
        Account access key.

        :param access_key: The access_key of this Bucket.
        :type: str
        """

        self._access_key = access_key

    @property
    def endpoint_url(self):
        """
        Endpoint URL for bucket.

        :return: The endpoint_url of this Bucket. 
        :rtype: str
        """
        return self._endpoint_url

    @endpoint_url.setter
    def endpoint_url(self, endpoint_url):
        """
        Endpoint URL for bucket.

        :param endpoint_url: The endpoint_url of this Bucket.
        :type: str
        """

        self._endpoint_url = endpoint_url

    @property
    def id(self):
        """
        Unique integer identifying a bucket.

        :return: The id of this Bucket. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying a bucket.

        :param id: The id of this Bucket.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Bucket name.

        :return: The name of this Bucket. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Bucket name.

        :param name: The name of this Bucket.
        :type: str
        """

        self._name = name

    @property
    def organization(self):
        """
        Unique integer identifying organization that owns this bucket.

        :return: The organization of this Bucket. 
        :rtype: int
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """
        Unique integer identifying organization that owns this bucket.

        :param organization: The organization of this Bucket.
        :type: int
        """

        self._organization = organization

    @property
    def region(self):
        """
        Bucket region.

        :return: The region of this Bucket. 
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """
        Bucket region.

        :param region: The region of this Bucket.
        :type: str
        """

        self._region = region

    @property
    def secret_key(self):
        """
        Account secret key.

        :return: The secret_key of this Bucket. 
        :rtype: str
        """
        return self._secret_key

    @secret_key.setter
    def secret_key(self, secret_key):
        """
        Account secret key.

        :param secret_key: The secret_key of this Bucket.
        :type: str
        """

        self._secret_key = secret_key

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Bucket):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Bucket):
            return True

        return self.to_dict() != other.to_dict()
